//  Basewindow.cc for bbtools.
//
//  Copyright (c) 1998-2000 by John Kennis, jkennis@chello.nl
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// (See the included file COPYING / GPL-2.0)
//

#include "bbmail.hh"
#include "main.hh"

void Usage()
{
	fprintf(stdout,"\n%s version %s \n",BBTOOL,BBTOOL_VERSION);
	fprintf(stdout,"Usage: %s [options]\n",BBTOOL);
	fprintf(stdout,"Options:\n");
	fprintf(stdout,"-display <display name>     X server to connect to\n");
	fprintf(stdout,"-c[onfig] <filename>        Alternate config file\n");
	fprintf(stdout,"-n[obb]                     Fall back on default configuration\n");
	fprintf(stdout,"-v[ersion]                  Display version number\n");
	fprintf(stdout,"-h[elp]                     Display this help\n");
	fprintf(stdout,"-geom[etry] <geometry>      Set geometry of window\n");
	fprintf(stdout,"-d[ecorated]                Show 'normal' decorated window\n");
	fprintf(stdout,"-w[ithdrawn]                Place bbtool in the Slit\n");
	fprintf(stdout,"-s[hape]                    Don't display groundplate\n");

	fprintf(stdout,"-sp[ooldir]                 Define alternate spooldir\n");
	fprintf(stdout,"-m[ailbox] <number>         Show only mailbox <number>\n\n");
}


int main(int argc,char **argv)
{
	int i;
	struct CMDOPTIONS options;
	options.display_name=NULL;
	options.geometry=NULL;
	options.withdrawn=False;
	options.shape=False;
	options.config_file=NULL;
	options.nobb_config=False;
	options.decorated=False;
	options.spooldir=NULL;
	options.mailbox=0;
	options.app_name=argv[0];


	for(i=1;i<argc;i++) {
		if ((!strcmp(argv[i],"-display"))) {
			if(++i==argc)  {
				Usage();
				exit(2);
			};
			options.display_name=argv[i];
		} else if ((!strcmp(argv[i],"-config"))|(!strcmp(argv[i],"-c"))) {
			if(++i==argc)  {
				Usage();
				exit(2);
			};
			options.config_file=argv[i];
		} else if ((!strcmp(argv[i],"-nobb"))|(!strcmp(argv[i],"-n"))) {
			options.nobb_config=True;
		} else if ( (!strcmp(argv[i],"-v")) || (!strcmp(argv[i],"-version"))) {
			fprintf(stderr," %s version %s\n",BBTOOL,BBTOOL_VERSION);
			exit(2);
		} else if ( (!strcmp(argv[i],"-h")) || (!strcmp(argv[i],"-help"))) {
			Usage();
			exit(2);
		} else if ((!strcmp(argv[i],"-geometry")) || (!strcmp(argv[i],"-geom"))) {
			if(++i==argc)  {
				Usage();
				exit(2);
			};
			options.geometry=argv[i];
		}
		else if ((!strcmp(argv[i],"-withdrawn")) || (!strcmp(argv[i],"-w"))) {
			options.withdrawn=True;
		} else if ((!strcmp(argv[i],"-shape")) || (!strcmp(argv[i],"-s"))) {
			options.shape=True;
		} else if ((!strcmp(argv[i],"-decorated")) || (!strcmp(argv[i],"-d"))) {
			options.decorated=True;
		} else if ((!strcmp(argv[i],"-spooldir")) || (!strcmp(argv[i],"-sp"))) {
			if(++i==argc)  {
				Usage();
				exit(2);
			};
			options.spooldir=argv[i];
		}
		if ((!strcmp(argv[i],"-m"))||(!strcmp(argv[i],"-mailbox"))) {
			if(++i==argc)  {
				Usage();
				exit(2);
			};
			if (sscanf(argv[i], "%d", &options.mailbox) != 1) {
				Usage();
				exit(2);
			};
		}
	}
	ToolWindow bbmail(argc,argv,&options);
}
